MODULE ArithmTest;	(** AUTHOR "SAGE"; PURPOSE "Testbed"; *)

IMPORT
	Types, Kernel, Clock, Files, Modules, Random, Inputs, Commands, Options, XML, Streams, Strings,
	WMWindowManager, WMStandardComponents, WMDialogs, WMEditors, WMComponents;

TYPE

	Test = OBJECT
	
		VAR
			nQuestions, nAnswersGood, nAnswersBad: LONGINT;
			sw: Streams.StringWriter;
			str: ARRAY 32 OF CHAR;
			
		PROCEDURE &Init;
		BEGIN
			nQuestions := 0;
			nAnswersGood := 0;
			nAnswersBad := 0;
			NEW(sw, 32)
		END Init;
		
		PROCEDURE Question(): BOOLEAN;
		BEGIN
			RETURN FALSE
		END Question;
		
		PROCEDURE QuestionText(): Strings.String;
		BEGIN
			RETURN NIL
		END QuestionText;
		
		PROCEDURE AnswerCheck(CONST value: Types.Any): BOOLEAN;
		BEGIN
			RETURN FALSE
		END AnswerCheck;
			
	END Test;
	
	Multiplication = OBJECT(Test)
	
		VAR
			iNum1, iNum2: LONGINT;
			table: ARRAY 10 OF ARRAY 10 OF RECORD
				bRequested: BOOLEAN
			END;
		
		PROCEDURE &Init;
		VAR
			i, j: LONGINT;
		BEGIN
			Init^;
			FOR j := 0 TO LEN(table, 0) - 1 DO
				FOR i := 0 TO LEN(table, 1) - 1 DO
					table[j][i].bRequested := FALSE
				END
			END
		END Init;
		
		PROCEDURE Question(): BOOLEAN;
		VAR
			b: BOOLEAN;
			i, j: LONGINT;
		BEGIN
			b := FALSE;
			WHILE ~b & (nQuestions < LEN(table, 0) * LEN(table, 1)) DO
				j := random.Dice(LEN(table, 0)) + 1;
				i := random.Dice(LEN(table, 1)) + 1;
				b := ~table[j - 1][i - 1].bRequested;
				IF b THEN
					INC(nQuestions);
					table[j - 1][i - 1].bRequested := TRUE;
					iNum1 := i;
					iNum2 := j
				END
			END;
			RETURN b
		END Question;
		
		PROCEDURE QuestionText(): Strings.String;
		BEGIN
			IF nQuestions > 0 THEN
				sw.String(" "); sw.Int(iNum1, 0); sw.String(" * "); sw.Int(iNum2, 0); sw.String(" ="); sw.Get(str);
				RETURN Strings.NewString(str)
			ELSE
				RETURN NIL
			END
		END QuestionText;
		
		PROCEDURE AnswerCheck(CONST value: Types.Any): BOOLEAN;
		VAR
			bAnswered: BOOLEAN;
			iValue, iRes: LONGINT;
		BEGIN
			Types.GetInteger(value, iValue, iRes);
			IF (iRes = Types.Ok) & (nQuestions > 0) THEN
				bAnswered := iNum1 * iNum2 = iValue;
				IF bAnswered THEN
					INC(nAnswersGood)
				ELSE
					INC(nAnswersBad)
				END;
				RETURN bAnswered
			ELSE
				RETURN FALSE
			END
		END AnswerCheck;
			
	END Multiplication;
	
	Division = OBJECT(Test)
	
		VAR
			iNum1, iNum2: LONGINT;
			table: ARRAY 10 OF ARRAY 10 OF RECORD
				bRequested: BOOLEAN
			END;
		
		PROCEDURE &Init;
		VAR
			i, j: LONGINT;
		BEGIN
			Init^;
			FOR j := 0 TO LEN(table, 0) - 1 DO
				FOR i := 0 TO LEN(table, 1) - 1 DO
					table[j][i].bRequested := FALSE
				END
			END
		END Init;
		
		PROCEDURE Question(): BOOLEAN;
		VAR
			b: BOOLEAN;
			i, j: LONGINT;
		BEGIN
			b := FALSE;
			WHILE ~b & (nQuestions < LEN(table, 0) * LEN(table, 1)) DO
				j := random.Dice(LEN(table, 0)) + 1;
				i := random.Dice(LEN(table, 1)) + 1;
				b := ~table[j - 1][i - 1].bRequested;
				IF b THEN
					INC(nQuestions);
					table[j - 1][i - 1].bRequested := TRUE;
					iNum1 := i;
					iNum2 := j
				END
			END;
			RETURN b
		END Question;
		
		PROCEDURE QuestionText(): Strings.String;
		BEGIN
			IF nQuestions > 0 THEN
				sw.String(" "); sw.Int(iNum1 * iNum2, 0); sw.String(" : "); sw.Int(iNum1, 0); sw.String(" ="); sw.Get(str);
				RETURN Strings.NewString(str)
			ELSE
				RETURN NIL
			END
		END QuestionText;
		
		PROCEDURE AnswerCheck(CONST value: Types.Any): BOOLEAN;
		VAR
			bAnswered: BOOLEAN;
			iValue, iRes: LONGINT;
		BEGIN
			Types.GetInteger(value, iValue, iRes);
			IF (iRes = Types.Ok) & (nQuestions > 0) THEN
				bAnswered := iNum2 = iValue;
				IF bAnswered THEN
					INC(nAnswersGood)
				ELSE
					INC(nAnswersBad)
				END;
				RETURN bAnswered
			ELSE
				RETURN FALSE
			END
		END AnswerCheck;
			
	END Division;
	
	Addition = OBJECT(Test)
	
		VAR
			iNum1, iNum2: LONGINT;
			table: ARRAY 10 OF ARRAY 10 OF RECORD
				bRequested: BOOLEAN
			END;
		
		PROCEDURE &Init;
		VAR
			i, j: LONGINT;
		BEGIN
			Init^;
			FOR j := 0 TO LEN(table, 0) - 1 DO
				FOR i := 0 TO LEN(table, 1) - 1 DO
					table[j][i].bRequested := FALSE
				END
			END
		END Init;
		
		PROCEDURE Question(): BOOLEAN;
		VAR
			b: BOOLEAN;
			i, j: LONGINT;
		BEGIN
			b := FALSE;
			WHILE ~b & (nQuestions < LEN(table, 0) * LEN(table, 1)) DO
				j := random.Dice(LEN(table, 0)) + 1;
				i := random.Dice(LEN(table, 1)) + 1;
				b := ~table[j - 1][i - 1].bRequested;
				IF b THEN
					INC(nQuestions);
					table[j - 1][i - 1].bRequested := TRUE;
					iNum1 := i;
					iNum2 := j
				END
			END;
			RETURN b
		END Question;
		
		PROCEDURE QuestionText(): Strings.String;
		BEGIN
			IF nQuestions > 0 THEN
				sw.String(" "); sw.Int(iNum1, 0); sw.String(" + "); sw.Int(iNum2, 0); sw.String(" ="); sw.Get(str);
				RETURN Strings.NewString(str)
			ELSE
				RETURN NIL
			END
		END QuestionText;
		
		PROCEDURE AnswerCheck(CONST value: Types.Any): BOOLEAN;
		VAR
			bAnswered: BOOLEAN;
			iValue, iRes: LONGINT;
		BEGIN
			Types.GetInteger(value, iValue, iRes);
			IF (iRes = Types.Ok) & (nQuestions > 0) THEN
				bAnswered := iNum1 + iNum2 = iValue;
				IF bAnswered THEN
					INC(nAnswersGood)
				ELSE
					INC(nAnswersBad)
				END;
				RETURN bAnswered
			ELSE
				RETURN FALSE
			END
		END AnswerCheck;
			
	END Addition;

	Window = OBJECT(WMComponents.FormWindow)
	
		VAR
			test: Test;
			
			tfAnswer: WMEditors.TextField;
			lbQuestion, lbStatus, lbTimeLeft: WMStandardComponents.Label;
			
			timer: Kernel.Timer;
			bAlive, bTerminated, bQuestion, bAnswerCheck: BOOLEAN;
			nDuration, nTimeLeft: LONGINT;
			sr: Streams.StringReader;
			sw: Streams.StringWriter;
			s: Strings.String;
			str: ARRAY 32 OF CHAR;
			filename: ARRAY 16 OF CHAR;
			file: Files.File;
			out: Files.Writer;
			time, date: LONGINT;
		
		PROCEDURE &Create(width, height: LONGINT; alpha: BOOLEAN; test: Test; strTest: ARRAY OF CHAR; nDuration: LONGINT);
		BEGIN
			Init(width, height, alpha);
			SELF.test := test;
			SELF.nDuration := nDuration;
			SetTitle(Strings.NewString(strTest));
			NEW(sr, 32);
			NEW(sw, 32);
			NEW(timer);
			bQuestion := FALSE;
			bAlive := TRUE;
			bTerminated := FALSE;
			bAnswerCheck := FALSE;
			COPY("ArithmTest.Text", filename);
			file := Files.Old(filename);
			IF file # NIL THEN
				Files.OpenWriter(out, file, file.Length());
				out.Ln; out.Ln;
				out.Update
			ELSE
				file := Files.New(filename);
				IF file # NIL THEN
					Files.Register(file);
					Files.OpenWriter(out, file, 0)
				END
			END;
			IF out # NIL THEN
				Clock.Get(time, date);
				out.Date(time, date); out.Ln;
				out.String(strTest); out.Ln;
				out.Ln;
				out.Update
			END
		END Create;

		PROCEDURE Close;
		BEGIN
			bAlive := FALSE;
			timer.Wakeup;
			IF out # NIL THEN
				out.Ln;
				out.Update
			END;
			BEGIN{EXCLUSIVE}
				AWAIT(bTerminated)
			END;
			Close^;
			window := NIL
		END Close;
		
		PROCEDURE Question;
		BEGIN
			nTimeLeft := nDuration;
			bQuestion := test.Question();
			IF bQuestion THEN
				lbQuestion.caption.Set(test.QuestionText())
			ELSE
				lbQuestion.caption.Set(NIL);
				lbTimeLeft.caption.Set(NIL);
				tfAnswer.visible.Set(FALSE)
			END;
			tfAnswer.SetAsString("")
		END Question;
		
		PROCEDURE FormatStatus(nQ, nG, nB: LONGINT): Strings.String;
		BEGIN
			sw.String(" "); sw.Int(nQ, 0); sw.String(" (good: "); sw.Int(nG, 0); sw.String(", bad: "); sw.Int(nB, 0); sw.String(")"); sw.Get(str);
			RETURN Strings.NewString(str)
		END FormatStatus;
		
		PROCEDURE FormatTimeLeft(): Strings.String;
		BEGIN
			sw.String(" "); sw.Int(nTimeLeft, 0); sw.Get(str);
			RETURN Strings.NewString(str)
		END FormatTimeLeft;
				
		PROCEDURE AnswerCheck;
		VAR
			value: Types.Integer;
			iValue, iRes: LONGINT;
		BEGIN
			BEGIN{EXCLUSIVE}
				AWAIT(~bAnswerCheck);
				bAnswerCheck := TRUE;
			END;
			tfAnswer.GetAsString(str);
			sr.Set(str);
			sr.SkipWhitespace;
			sr.Int(iValue, FALSE);
			Types.SetInteger(value, iValue, iRes);
			IF (iRes = Types.Ok) & ~test.AnswerCheck(value) THEN
				IF out # NIL THEN
					s := test.QuestionText();
					out.String(s^); out.String(" @ "); out.String(str); out.Ln;
					out.Update
				END
			END;
			tfAnswer.SetAsString("");
			lbStatus.caption.Set(FormatStatus(test.nQuestions, test.nAnswersGood, test.nAnswersBad));
			Question;
			BEGIN{EXCLUSIVE}
				bAnswerCheck := FALSE
			END
		END AnswerCheck;
		
		PROCEDURE KeyEvent(ucs: LONGINT; flags: SET; keysym: LONGINT);
		BEGIN
			IF ((keysym = Inputs.KsReturn) OR (keysym = Inputs.KsKPEnter)) & bQuestion THEN
				AnswerCheck
			ELSIF keysym = Inputs.KsEscape THEN
				Close
			END
		END KeyEvent;
		
		PROCEDURE DecTimeLeft(): BOOLEAN;
		VAR
			b: BOOLEAN;
		BEGIN{EXCLUSIVE}
			b := FALSE;
			IF nTimeLeft > 0 THEN
				DEC(nTimeLeft);
				b := TRUE
			END;
			RETURN b
		END DecTimeLeft;
		
	BEGIN{ACTIVE}
		timer.Sleep(1000);
		Question;
		WHILE bQuestion & bAlive DO
			WHILE bQuestion & bAlive & DecTimeLeft() DO
				lbTimeLeft.caption.Set(FormatTimeLeft());
				timer.Sleep(1000)
			END;
			IF bQuestion & bAlive THEN
				AnswerCheck
			END
		END;
		BEGIN{EXCLUSIVE}
			bTerminated := TRUE
		END
	END Window;

VAR
	window: Window;
	random: Random.Generator;	
	
PROCEDURE FabMultiplication(): Test;
VAR
	multiplication: Multiplication;
BEGIN
	NEW(multiplication);
	RETURN multiplication
END FabMultiplication;

PROCEDURE FabDivision(): Test;
VAR
	division: Division;
BEGIN
	NEW(division);
	RETURN division
END FabDivision;

PROCEDURE FabAddition(): Test;
VAR
	addition: Addition;
BEGIN
	NEW(addition);
	RETURN addition
END FabAddition;

PROCEDURE Open*(context : Commands.Context); (** [Options] <RepositoryName:ComponentName:ID> | <ModuleName.ProcedureName> | <Filename> ~ *)
VAR
	options: Options.Options;
	filename, strTest: ARRAY 128 OF CHAR;
	c, c1, c2, c3, c4: XML.Content; 
	vc: WMComponents.VisualComponent;
	x, y: LONGINT;
	manager: WMWindowManager.WindowManager;
	test: Test;
	b: BOOLEAN;
	nDuration: LONGINT;
BEGIN
	IF window = NIL THEN
		NEW(options);
		options.Add("a","alpha", Options.Flag);
		options.Add("t","test", Options.String);
		options.Add("d","duration", Options.Integer);
		IF options.Parse(context.arg, context.error) & context.arg.GetString(filename) THEN
			(* Load component from XML file *)
			c := WMComponents.Load(filename);
			IF (c # NIL) & (c IS WMComponents.VisualComponent) THEN
				vc := c(WMComponents.VisualComponent);
			
				c1 := vc.FindByUID("tfAnswer");
				c2 := vc.FindByUID("lbQuestion");
				c3 := vc.FindByUID("lbStatus");
				c4 := vc.FindByUID("lbTimeLeft");
				
				IF (c1 # NIL) & (c1 IS WMEditors.TextField) &
					(c2 # NIL) & (c2 IS WMStandardComponents.Label) &
					(c3 # NIL) & (c3 IS WMStandardComponents.Label) &
					(c4 # NIL) & (c4 IS WMStandardComponents.Label)
				THEN
					test := NIL;
					IF options.GetString("test", strTest) THEN
						IF strTest = "Multiplication" THEN
							test := FabMultiplication()
						ELSIF strTest = "Division" THEN
							test := FabDivision()
						ELSIF strTest = "Addition" THEN
							test := FabAddition()
						END;
						IF test # NIL THEN
							
							b := options.GetInteger("duration", nDuration);
							IF nDuration < 5 THEN nDuration := 5 END;
							
							random.InitSeed(Kernel.GetTicks());
							NEW(window, vc.bounds.GetWidth(), vc.bounds.GetHeight(), options.GetFlag("alpha"), test, strTest, nDuration);
							
							window.tfAnswer := c1(WMEditors.TextField);
							window.lbQuestion := c2(WMStandardComponents.Label);
							window.lbStatus := c3(WMStandardComponents.Label);
							window.lbTimeLeft := c4(WMStandardComponents.Label);
							
							window.SetContent(vc);
							WMDialogs.DefaultPos(x, y, window.GetWidth(), window.GetHeight());
							WMWindowManager.AddWindow(window, x, y);
							manager := WMWindowManager.GetDefaultManager();
							manager.SetFocus(window)

						END
					END
				ELSE
					context.error.String("Wrong form definition "); context.error.String(filename);
				END
			ELSIF c = NIL THEN context.error.String("Could not load/generate component "); context.error.String(filename);
			ELSE
			END
		ELSE
			context.error.String("Usage: MultTest.Open [Options] MultTest.XML ~"); context.error.Ln
		END
	END
END Open;

PROCEDURE Cleanup;
BEGIN
	IF window # NIL THEN
		window.Close
	END
END Cleanup;

BEGIN
	NEW(random);
	Modules.InstallTermHandler(Cleanup)
END ArithmTest.

ArithmTest.Open --alpha -t=Multiplication -d=10 ArithmTest.XML ~
ArithmTest.Open --alpha -t=Division -d=10 ArithmTest.XML ~
ArithmTest.Open --alpha -t=Addition -d=10 ArithmTest.XML ~

SystemTools.Free ArithmTest ~



